from typing import Type, Tuple
from pathlib import Path

from pydantic import BaseModel
from pydantic_settings import (
    BaseSettings,
    # SecretsSettingsSource,
    SettingsConfigDict,
    PydanticBaseSettingsSource,
    YamlConfigSettingsSource,
)

config_files = [Path(Path.cwd().root) / "etc" / "gacsco" / "congig.yaml"]


class Connection(BaseModel):
    url: str = "http://localhost:7557"
    username: str = "admin"
    password: str = "admin"


class Config(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="GACSCO__",
        nested_model_default_partial_update=True,
        env_nested_delimiter="__",
        # secrets_dir=("/var/run", "/run/secrets", Path.cwd() / "secrets"),
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        configs = [
            YamlConfigSettingsSource(settings_cls, yaml_file=cf) for cf in config_files
        ]
        return (
            env_settings,
            init_settings,
            file_secret_settings,
            dotenv_settings,
            *configs,
        )

    connection: Connection = Connection()


conf = Config()
